import "./quizBot.css";
import { AnswerButton } from "../../../../components/quiz/answerButton/answerButton.tsx";
import { ValidationButton } from "../../../../components/quiz/validationButton/validationButton.tsx";
import { useEffect, useState } from "react";
import HttpError from "../../../../services/HttpError.ts";
import { useMultiplayerService } from "../../../../services/MultiplayerService.ts";
import { Answer } from "../../../../models/Answer.ts";

interface Props {
    question: any;
}

enum QuizState {
    ANSWERING = "Answering",
    LOADING = "Loading",
    SHOWING_RESULTS = "ShowingResults",
}

const getStyleOfAnswerButton = (
    answerId: number,
    selectedAnswerIndex: number | null,
    correctAnswerId: number | null,
    quizState: QuizState
): string => {
    if (quizState === QuizState.LOADING) {
        return "answer-button answer-button-loading";
    }
    if (quizState === QuizState.ANSWERING) {
        if (selectedAnswerIndex === null) return "answer-button";
        if (selectedAnswerIndex === answerId) return "answer-button answer-button-selected";
    }
    if (quizState === QuizState.SHOWING_RESULTS) {
        if (answerId === correctAnswerId) return "answer-button answer-button-correct";
        if (answerId !== correctAnswerId && selectedAnswerIndex === answerId)
            return "answer-button answer-button-wrong";
    }
    return "answer-button";
};
export function QuizBot({ question }: Props) {
    const [questionIsFinished, setQuestionIsFinished] = useState(false);
    const [selectedAnswerId, setSelectedAnswerId] = useState<number | null>(null);
    const [validationButtonIsDisabled, setValidationButtonIsDisabled] = useState(true);
    const [quizState, setQuizState] = useState(QuizState.ANSWERING);
    const [correctAnswerId, setCorrectAnswerId] = useState<number | null>(null);

    const { sendAnswer, recivedResults, recivedQuestion } = useMultiplayerService();

    useEffect(() => {
        setSelectedAnswerId(null); // Réinitialise la sélection
        setQuestionIsFinished(false); // Réinitialise l'état de la question
        setCorrectAnswerId(null); // Réinitialise l'id de la bonne réponse
        setQuizState(QuizState.ANSWERING);
    }, [question.questionIndex]);

    const getCorrectAnswer = (answers: Answer[]): Answer => {
        const correctAnswer = answers.find((answer) => answer.isCorrect);
        if (!correctAnswer) throw new Error("No correct answer found");
        return correctAnswer;
    };
    const onValidation = async () => {
        setQuizState(QuizState.LOADING);
        const answerId = question.answers.find((answer) => answer.id === selectedAnswerId)?.id;
        if (!answerId) return;
        sendAnswer(question.id_quiz, question.id, answerId);
        const answers = await recivedResults();
        if (HttpError.isHttpError(answers)) {
            return;
        }
        setQuizState(QuizState.SHOWING_RESULTS);
        const correctAnswerIdFetched = getCorrectAnswer(answers).id;
        setCorrectAnswerId(correctAnswerIdFetched);
    };

    const onContinue = () => {
        question = recivedQuestion();
    };

    useEffect(() => {
        if (selectedAnswerId !== null) {
            setValidationButtonIsDisabled(false);
            return;
        }
        setValidationButtonIsDisabled(true);
    }, [selectedAnswerId]);

    const onAnsweredButtonClicked = (answerId: number) => {
        if (quizState === QuizState.SHOWING_RESULTS || quizState === QuizState.LOADING) {
            return;
        }
        setSelectedAnswerId(answerId);
    };

    return (
        <div className="quiz-bot-container">
            <div className="quiz-bot-button-container">
                {question.answers.map((answer) => {
                    return (
                        <>
                            <p>{quizState === QuizState.SHOWING_RESULTS ? answer.nb_answers : null}</p>
                            <AnswerButton
                                text={answer.text}
                                key={answer.id}
                                onClick={() => onAnsweredButtonClicked(answer.id)}
                                style={getStyleOfAnswerButton(answer.id, selectedAnswerId, correctAnswerId, quizState)}
                                isDisabled={quizState === QuizState.LOADING}
                            />
                        </>
                    );
                })}
            </div>
            <div className="quiz-bot-validation-button-container">
                <ValidationButton
                    text={quizState === QuizState.ANSWERING ? "CONFIRM" : "CONTINUE"}
                    onClick={() => (quizState === QuizState.ANSWERING ? onValidation() : onContinue())}
                    isDisabled={validationButtonIsDisabled}
                />
            </div>
        </div>
    );
}
