import "./quizBot.css";
import {AnswerButton} from "../../../components/quiz/answerButton/answerButton.tsx";
import {ValidationButton} from "../../../components/quiz/validationButton/validationButton.tsx";
import {Dispatch, SetStateAction, useEffect, useState} from "react";
import {useQuizService} from "../../../services/QuizService.ts";
import HttpError from "../../../services/HttpError.ts";
import {Answer} from "../../../models/Answer.ts";
import {Question} from "../../../models/Question.ts";
import {Quiz} from "../../../models/Quiz.ts";
import {QuizInformations} from "../../../models/QuizInformations.ts";

interface Props {
    quizInformations: QuizInformations;
    runId?: string;
    actualQuestion: Question;
    fetchActualQuestion: () => void;
    setScore: (score: number) => void;
    OpenModalEndQuiz: (bool: boolean) => void;
}

enum QuizState {
    ANSWERING= "Answering",
    LOADING= "Loading",
    SHOWING_RESULTS= "ShowingResults",
}

const getStyleOfAnswerButton = (answerId: number, selectedAnswerIndex: number | null, correctAnswerId: number | null, quizState: QuizState) : string => {
    if(quizState === QuizState.LOADING)
    {
        return "answer-button answer-button-loading";
    }
    if(quizState === QuizState.ANSWERING)
    {
        if(selectedAnswerIndex === null) return "answer-button";
        if(selectedAnswerIndex === answerId) return "answer-button answer-button-selected";

    }
    if(quizState === QuizState.SHOWING_RESULTS)
    {
        if(answerId === correctAnswerId) return "answer-button answer-button-correct";
        if(answerId !== correctAnswerId && selectedAnswerIndex === answerId) return "answer-button answer-button-wrong";
    }
    return "answer-button"

}
export function QuizBot({quizInformations, runId, actualQuestion, fetchActualQuestion, setScore, OpenModalEndQuiz} : Props) {

    const [selectedAnswerId, setSelectedAnswerId] = useState<number | null>(null);
    const [quizState, setQuizState] = useState(QuizState.ANSWERING);
    const [correctAnswerId, setCorrectAnswerId] = useState<number | null>(null);
    const [isLastQuestion, setIsLastQuestion] = useState(false);

    const {answerQuestion} = useQuizService();

    const getCorrectAnswer = (answers: Answer[]): Answer => {
        const correctAnswer = answers.find((answer) => answer.isCorrect);
        if (!correctAnswer) throw new Error("No correct answer found");
        return correctAnswer;
    };
    const onValidation = async () => {
        if(selectedAnswerId === null) return;

        setQuizState(QuizState.LOADING);
        const answerId = actualQuestion.answers.find((answer) => answer.id === selectedAnswerId)?.id;

        if(!answerId || !runId) return;
        const answereFetched = await answerQuestion(runId, actualQuestion.id, answerId);


        if (HttpError.isHttpError(answereFetched)) {
            return;
        }

        setScore(answereFetched.score);
        const correctAnswerIdFetched = getCorrectAnswer(answereFetched.answers.answers).id;

        setCorrectAnswerId(correctAnswerIdFetched);
        setQuizState(QuizState.SHOWING_RESULTS);

        // Check si c'est la dernière question
        if(answereFetched.answers.order === quizInformations.questionCount - 1) {
            setIsLastQuestion(true);
            return;
        }
    };

    const onContinue = () => {
        if(isLastQuestion) {
            OpenModalEndQuiz(true);
            return;
        }
        fetchActualQuestion();
        setQuizState(QuizState.ANSWERING);
        setSelectedAnswerId(null);
        setCorrectAnswerId(null);
    }

    const onAnsweredButtonClicked = (answerId: number) => {
        if(quizState === QuizState.SHOWING_RESULTS || quizState === QuizState.LOADING) {
            return;
        }
        setSelectedAnswerId(answerId);
    }


    return (
        <div className="quiz-bot-container">
            <div className="quiz-bot-button-container">
                {
                    actualQuestion.answers.map((answer) => {
                        return <AnswerButton text={answer.text} key={answer.id} onClick={() => onAnsweredButtonClicked(answer.id)} style={getStyleOfAnswerButton(answer.id, selectedAnswerId, correctAnswerId, quizState)} isDisabled={quizState === QuizState.LOADING} />
                    })
                }
            </div>
            <div className="quiz-bot-validation-button-container">
                <ValidationButton text={quizState === QuizState.ANSWERING ? "CONFIRM" : "CONTINUE"} onClick={() => quizState === QuizState.ANSWERING ? onValidation() : onContinue()} isDisabled={false}/>
            </div>
        </div>
    );
}
