import HttpError from "./HttpError";
import axios from "axios";
import { Quiz, QuizCommunityResponse, QuizCreation } from '../models/Quiz.ts';
import { Answer } from '../models/Answer.ts';
import { AnswerResponse } from '../models/AnswerResponse.ts';
import { Question } from '../models/Question.ts';
import { QuestionResponse } from '../models/QuestionResponse.ts';
import { QuizInformations } from '../models/QuizInformations.ts';
import { QuizGenerateAnswer } from '../models/QuizGenerateAnswer.ts';

export const useQuizService = () => {
    const url = 'https://klebert-host.com:33037'; // Remplace par l'URL de ton endpoint API

    const generateQuiz = async (
        amount: number,
        category: number,
        difficulty: string
    ): Promise<QuizGenerateAnswer | HttpError> => {
        const requestBody = {
            amount,
            category,
            difficulty: difficulty.toLowerCase(),
        };

        try {
            const response = await axios.post<QuizGenerateAnswer>(`${url}/quiz/generate`, requestBody, {
                headers: {
                    'Content-Type': 'application/json',
                },
            });
            const runId = response.data.id;
            return { id: runId, quizId: response.data.quizId };
        } catch (error: any) {
            console.log('Error while generating quiz:', error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const getRandomQuiz = async (): Promise<QuizGenerateAnswer | HttpError> => {
        const requestBody = {
            amount: 10,
            category: 10,
            difficulty: 'easy',
        };

        try {
            const response = await axios.post<QuizGenerateAnswer>(`${url}/quiz/generate`, requestBody, {
                headers: {
                    'Content-Type': 'application/json',
                },
            });
            const runId = response.data.id;
            // Récupération du run ID
            return { id: runId, quizId: response.data.quizId };
        } catch (error: any) {
            console.log('Error while generating quiz:', error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const getCommunityQuiz = async (skip: number, take: number, search: string) => {
        try {
            const response = await axios.get<QuizCommunityResponse>(`${url}/quiz/paginated?skip=${skip}&take=${take}&search=${search}`, {
                withCredentials: true, // Inclut les cookies pour une session authentifiée
            });

            return { quizzes: response.data.quizzes, total: response.data.total };
        } catch (error: any) {
            console.error('Error while fetching all quizzes:', error);

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const createCommunityQuiz = async (quiz: QuizCreation): Promise<boolean | HttpError> => {
        try {
            await axios.post(`${url}/quiz/community`, quiz, {
                withCredentials: true,
            });
            return true;
        } catch (error: any) {
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };
    const getCategories = async () => {
        try {
            const response = await axios.get(`${url}/quiz/categories`, {
                headers: {
                    'Content-Type': 'application/json',
                },
            });
            return response.data;
        } catch (error: any) {
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const answerQuestion = async (
        quizId: string,
        questionId: number,
        answerId: number
    ): Promise<AnswerResponse | HttpError> => {
        const requestBody = {
            id: quizId,
            questionID: questionId,
            answerID: answerId,
        };
        try {
            const response = await axios.post<AnswerResponse>(`${url}/run/answer`, requestBody, {
                headers: {
                    'Content-Type': 'application/json',
                },
            });

            return response.data;
        } catch (error: any) {
            console.log('Error while remaining quiz:', error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const getActualQuestionOfAQuiz = async (quizId: string): Promise<Question | HttpError> => {
        const requestBody = {
            id: quizId,
        };

        try {
            const response = await axios.post<QuestionResponse>(`${url}/run/question`, requestBody, {
                headers: {
                    'Content-Type': 'application/json',
                },
            });

            return response.data.question;
        } catch (error: any) {
            console.log('Error while remaining quiz:', error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const getQuizInformations = async (quizId: string): Promise<QuizInformations | HttpError> => {
        try {
            const response = await axios.get<QuizInformations>(`${url}/quiz/info/` + quizId, {
                headers: {
                    'Content-Type': 'application/json',
                },
            });
            return response.data;
        } catch (error: any) {
            console.log('Error while getQuizInformations quiz:', error);

            if (error.response) {
                // Gestion des erreurs HTTP
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                // Gestion des erreurs réseau ou autres
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const getQuestionsFromDatabase = async (category: number, difficulty: string, amount: number) => {
        try {
            const response = await axios.post(
                `${url}/question/importQuestions`,
                {
                    category,
                    difficulty,
                    amount,
                },
                {
                    withCredentials: true,
                    headers: {
                        'Content-Type': 'application/json',
                    },
                }
            );
            return response.data;
        } catch (error: any) {
            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const getQuizForUser = async (userId: number) => {
        try {
            const response = await axios.get(`${url}/quiz/user/${userId}`, {
                withCredentials: true,
            });
            return response.data;
        } catch (error: any) {
            console.error('Error while fetching all quizzes:', error);

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || 'HTTP error');
            } else {
                return new HttpError(500, 'Unexpected error: ' + error.message);
            }
        }
    };

    const createRun = async (quizId: string): Promise<string | HttpError> => {
        try {
            const response = await axios.post(`${url}/run/start`, {
                quizId: quizId,
            }, {
                headers: {
                    "Content-Type": "application/json",
                },
            });

            return response.data.id;
        } catch (error: any) {
            console.log("Error while getQuizInformations quiz:", error);

            if (error.response) {
                return new HttpError(error.response.status, error.response.data?.message || "HTTP error");
            } else {
                return new HttpError(500, "Unexpected error: " + error.message);
            }
        }
    }

    return {
        generateQuiz: generateQuiz,
        getRandomQuiz: getRandomQuiz,
        getCommunityQuiz: getCommunityQuiz,
        createCommunityQuiz: createCommunityQuiz,
        getCategories: getCategories,
        answerQuestion: answerQuestion,
        getActualQuestionOfAQuiz: getActualQuestionOfAQuiz,
        getQuizInformations: getQuizInformations,
        getQuestionsFromDatabase: getQuestionsFromDatabase,
        getQuizForUser,
       createRun: createRun,
    };
};
